# MT::Plugins::Redland::CMS::Topic
# -----------------
# $Revision: 1.2 $
# $Date: 2005/04/03 22:19:35 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic;

use strict;
use warnings;

use Carp qw(confess);
use IO::All;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data);
BEGIN {
	no warnings 'numeric';
	$NAME		= "Redland Extry Topic";
	$VERSION	= '0.001';
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds review statements to the Redland model',
		doc_link	=> 'http://kasei.us/code/mtsemweb/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms.pl' );
	require $cms;
}

our $debug		= 0;

######################################################################

MT::Plugins::Redland::CMS->install_cms_header( $NAME, sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $mode	= $q->param('__mode') || '';
	my $type	= $q->param('_type') || '';
	if ($type eq 'entry' and $mode eq 'view') {
		my $cfg		= MT::ConfigMgr->instance;
		my $path	= $cfg->StaticWebPath || $app->path;
		return <<"END";
<script type="text/javascript" src="${path}easytoggle3.js"></script>
END
	}
} );

MT::Plugins::Redland::CMS->install_cms_box( $NAME, sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $mode	= $q->param('__mode') || '';
	my $type	= $q->param('_type') || '';
	if ($type eq 'entry' and $mode eq 'view') {
		my @first;
		my @rest;
		my $count	= 1;
		our %topics;
		while (my($k,$v) = each(%topics)) {
			my $body	= ref($v) ? $v->( $app ) : $v;
			push(@first, qq(<li><a class="toggle" title="${k}" href="#${k}">${k}</a></li>));
			push(@rest, qq(<div id="${k}">${body}</div>));
		} continue { $count++ }
		
		my $body	= qq(<div><label>Entry Topic</label></div>\n)
					. qq(<div id="toggle"><ul class="toggle">)
					. join("\n", @first)
					. qq(</ul></div>\n)
					. qq(<div id="togglebox">) . join("\n", @rest) . qq(</div>\n);
		return $body;
	}
} );

######################################################################

sub install_topic {
	my $class	= shift;
	my $name	= shift;
	my $body	= shift;
	our %topics;
	$topics{ $name }	= $body;
}

sub topic_of_type {
	my $entry	= shift;
	my $type	= shift;
	my @topics	= topics( $entry );
	my $model	= $entry->driver->model;
	my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
	foreach my $topic (@topics) {
		warn "topic: " . $topic->as_string if ($debug);
		my $st	= RDF::Redland::Statement->new( $topic, $p_type, $type );
		warn $st->as_string if ($debug);
		if ($model->contains_statement( $st )) {
			return $topic;
		}
	}
	
}

sub topics {
	my $entry	= shift;
	my $model	= $entry->driver->model;
	my $node	= $entry->get_rdf_node;
	my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
	my $t_book	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/Book' );
	my $p_isbn	= RDF::Redland::Node->new_from_uri( 'http://purl.org/net/schemas/book/isbn' );
	my @topics	= $model->targets( $node, $p_topic );
	return @topics;
}

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic.pl,v $
 Revision 1.2  2005/04/03 22:19:35  greg
 - topic CMS hooks now only run if on the edit entry page
 - added a foaf:Document topic plugin

 Revision 1.1  2005/04/03 01:07:41  greg
 - added redland CMS Topic plugin
 - moved redland_cms_book.pl to redland_cms_topic_book.pl, now uses Topic API
 - added redland_cms_topic_event.pl for Event topics


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

=over 4

=item * Make Topic tag selected if there's an existing topic

=back

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
