# MT::Plugins::Redland::CMS::Topic::Event
# -----------------
# $Revision: 1.1 $
# $Date: 2005/04/03 01:07:41 $
# ---------------------------------------------------------

package MT::Plugins::Redland::CMS::Topic::Event;

use strict;
use warnings;
no warnings 'once';

use Carp qw(confess);
use IO::All;
use File::Spec;
use Data::Dumper;
use RDF::Redland;

use MT 2.6;
use MT::Template::Context;

our ($NAME, $VERSION, $plugin_data);
BEGIN {
	no warnings 'numeric';
	$NAME		= "Redland Event Topic";
	$VERSION	= '0.001';
	$plugin_data	= {
		name		=> "${NAME}, v${VERSION}",
		description => 'Adds an Event topic to an Entry',
		doc_link	=> 'http://kasei.us/code/mtredland/'
	};
	
	if ($MT::VERSION >= 3.0) {
		require MT::Plugin;
		MT::Plugin->import();
		our $plugin	= new MT::Plugin($plugin_data);
		MT->add_plugin($plugin);
	}
	my $cfg		= MT::ConfigMgr->instance;
	my $cms		= File::Spec->catfile( $cfg->PluginPath, 'redland_cms_topic.pl' );
	require $cms;
}

our $debug		= 0;

######################################################################

MT::Plugins::Redland::CMS->install_cms_header( ${NAME}, <<"END" );
<style type="text/css">
	#Event input { width: 300px; }
</style>
END
MT::Plugins::Redland::CMS::Topic->install_topic( 'Event', sub {
	my $app		= shift;
	my $q		= $app->{query};
	my $id		= $q->param('id');
	my $type	= $q->param('_type');
	my $class	= $app->_load_driver_for($type) or return;
	my $url		= '';
	my $title	= '';
	if ($id) {          # object exists, we're just editing it.
		my $entry	= $class->load($id) or return $app->error($app->translate("Load failed: [_1]", $class->errstr || "(no reason given)"));
		$url		= $entry->event_url;
		$title		= $entry->event_title;
	}
	return qq(<p><label for="event_title">Title</label>:<br/><input name="event_title" id="event_title" maxlength="255" value="${title}" /></p>\n)
		. qq(<p><label for="event_url">Homepage</label>:<br/><input name="event_url" id="event_url" maxlength="255" value="${url}" /></p>\n)
});

no warnings 'redefine';
my $orig	= MT::Entry->can('column_names');
*MT::Entry::column_names	= sub {
	my $self	= shift;
	my $names	= $orig->( $self, @_ );
	push(@{ $names }, qw(event_url event_title));
	return $names;
};

MT::Entry->add_callback("post_load", 1, $plugin_data, sub {
	my ($cb, $terms, $args, $entry) = @_;
	my $model	= $entry->driver->model;
	my $t_event		= RDF::Redland::Node->new_from_uri( 'http://www.cyc.com/2004/06/04/cyc#Event' );
	my $p_title		= RDF::Redland::Node->new_from_uri( 'http://purl.org/dc/elements/1.1/title' );
	my $p_homepage	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/homepage' );
	my $topic		= MT::Plugins::Redland::CMS::Topic::topic_of_type( $entry, $t_event );
	if ($topic) {
		my ($url)	= $model->targets( $topic, $p_homepage );
		my ($title)	= $model->targets( $topic, $p_title );
		$entry->event_url( $url->uri->as_string ) if ($url);
		$entry->event_title( $title->literal_value ) if ($title);
	}
});

MT::Entry->add_callback("pre_save", 5, $plugin_data, sub {
	my ($cb, $entry)	= @_;
	my $model	= $entry->driver->model;
	if ($entry->{'column_values'}{'event_url'} or $entry->{'column_values'}{'event_title'}) {
		my $url		= delete $entry->{'column_values'}{'event_url'};
		my $title	= delete $entry->{'column_values'}{'event_title'};
		my $node	= $entry->get_rdf_node;
		my $t_event		= RDF::Redland::Node->new_from_uri( 'http://www.cyc.com/2004/06/04/cyc#Event' );
		my $p_title		= RDF::Redland::Node->new_from_uri( 'http://purl.org/dc/elements/1.1/title' );
		my $p_homepage	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/homepage' );
		my $p_topic	= RDF::Redland::Node->new_from_uri( 'http://xmlns.com/foaf/0.1/topic' );
		my $p_type	= RDF::Redland::Node->new_from_uri( "http://www.w3.org/1999/02/22-rdf-syntax-ns#type" );
		
		# XXX should only remove topics of type cyc:Event
		my $st		= RDF::Redland::Statement->new( $node, $p_topic, undef );
		my @st		= $model->find_statements( $st );
		$model->remove_statement( $_ ) for (@st);
		
		my $event;
		if ($url) {
			my $r_url	= RDF::Redland::Node->new_from_uri( $url );
			($event)	= $model->sources( $p_homepage, $r_url );
		}
		unless ($event) {
			$event	= RDF::Redland::Node->new_from_blank_identifier('r' . time . 'r' . $MT::ObjectDriver::Redland::blankseq++);
			$model->add_statement( RDF::Redland::Statement->new( $event, $p_type, $t_event ) );
		}
		$st->object( $event );
		$model->add_statement( $st );
		if ($url) {
			my $st		= RDF::Redland::Statement->new( $event, $p_homepage, undef );
			my @st		= $model->find_statements( $st );
			$model->remove_statement( $_ ) for (@st);
			my $r_url	= RDF::Redland::Node->new_from_uri( $url );
			$st->object( $r_url );
			$model->add_statement( $st );
		}
		if ($title) {
			my $st		= RDF::Redland::Statement->new( $event, $p_title, undef );
			my @st		= $model->find_statements( $st );
			$model->remove_statement( $_ ) for (@st);
			my $l_title	= RDF::Redland::Node->new_literal( $title );
			$st->object( $l_title );
			$model->add_statement( $st );
		}
	}
});

######################################################################

1;

__END__

=head1 REVISION HISTORY

 $Log: redland_cms_topic_event.pl,v $
 Revision 1.1  2005/04/03 01:07:41  greg
 - added redland CMS Topic plugin
 - moved redland_cms_book.pl to redland_cms_topic_book.pl, now uses Topic API
 - added redland_cms_topic_event.pl for Event topics


=head1 SEE ALSO

 L<MT::Plugin>

=head1 KNOWN BUGS

None.

=head1 TODO

None.

=head1 LICENSE

Copyright (c) 2005 Gregory Williams. All rights reserved. This program is free
software; you can redistribute it and/or modify it under the same terms as Perl
itself.

=head1 AUTHOR

 Gregory Williams <gwilliams@cpan.org>

=cut
